import unittest
from rsgis import Metadata
import os

class MetadataTest(unittest.TestCase):

    def test_file(self):

        dir=os.getcwd() + '\data'

        for _ in os.listdir(dir):

            path = dir + '\\' + _

            if _.endswith('txt') and _.split('.')[0].endswith('_MTL'):

                try:

                    with open(path,'r') as file:

                        if file.readlines():

                            self.assertEqual(Metadata(path).path,path)

                except Exception as err:
                    raise err






if __name__ == '__main__':
    unittest.main()




import argparse

import metadata

parser=argparse.ArgumentParser(description='Get metadata value of query parameters')

parser.add_argument('--dir_path',type=str,help='Path to the directory of the metadata file', required=True)

parser.add_argument('--get', type=str,help='Query to execute. If query parameter is found,it return the value/dict of values')

args=parser.parse_args()

mtd=metadata.Metadata(args.dir_path)

mtd.get_cli(args.get)






"""
******************************************
MODULE FOR METADATA FILE HANDLING
******************************************
Author1: Jolaiya Emmanuel -> jolaiyaemmanuel@gmail.com
Author2: Oke Matthew -> matthewoke16@gmail,com
License: Apache
******************************************
"""

########
# HOW TO USE

"""
To get a particular parameter from the metadata file:
 
 #import
 from rsgis import Metadata
 mtd=Metadata('path\to\downloaded\landsat\file')
 
mtd.get('Solar Angle') = Returns the solar angle value in the metadata file
 
 metadata.get_some(['Solar Angle','RADIANCE']) = Return a list of the queried values in the metadata file
 
 metadata.get_all() = Return a dictionary of all the parameters in the metadata file
 
 
"""

import os

class Metadata:
    """
        This class handles the metadata files parameter access and query.

        Args:

         path(file path:str): path to metadata file

        exception:
         KeyError,TypeError

    """

    def __init__(self,path):

        self.path = path

    def _utility(self) -> dict:

        """Utility class to read metadata file and convert to dictionary"""

        #LANDSAT
        file=os.path.basename(self.path)
        assert isinstance(file, str), 'path must be of type string'

        assert 'LC0' in file.split('.')[0],'This is not a landsat metadata file. Check your file and retry'

        assert file.lower().endswith('.txt'), 'Landsat Metadata file must be in .txt(text) format'

        if file is not None and os.path.exists(self.path):

            try:
                # print('*_*' * 30)
                # print('\t\t\t\t\t -------------> LANDSAT METADATA <------------')

                metadata_dict = {}
                with open(self.path, mode='r') as metadata:
                    # assert metadata.readlines(),'File could not be opened. Pass in a valid metadata file'
                    for parameters in metadata.readlines():
                        parameters_list = parameters.split('=')
                        param_cleaned = ''.join(parameters_list[0].strip())
                        value_cleaned = ''.join(parameters_list[-1].strip())
                        metadata_dict[param_cleaned] = value_cleaned
                return metadata_dict

            except Exception as err:
                print(
                    'An error just occurred. Check the file to be sure it is a valid metadata file and check the parameters for possible error'
                    '\n You can also try the get_all() method to see'
                    'all the metadata parameters ')

                raise err

    def _incomplete_str_support(self, param:'str', metadata_dict:'dict') -> str:

        """
        A private method to guess incomplete query strings
        :param param:
        :param metadata_dict:
        :return: str or/and dict of guesses
        """

        try:

            assert isinstance(param, str), 'parameter must be of type str'

            multi_keys = {}

            for text in metadata_dict.keys():

                if param.upper() in text:
                    multi_keys[text] = metadata_dict.get(text)

            if len(multi_keys) > 1:

                print(
                    f'It seems your query **{param}** matches several parameters.Try to be  more specific to access your desired value, \n however, you can still make use of my guesses below')

                return multi_keys


            elif len(multi_keys) == 1:

                guess = list(multi_keys.keys())[0]

                print(
                    f'It seems you entered an incomplete query parameter. Was it **{guess}** you meant? If yes, see the result below.\n If no, try the get_all() method to see all'
                    f' available parameters.')

                return multi_keys.get(guess)

            else:

                return 'Could not find a match for your parameter. Try the .get_all() method to see available parameters \n or be more specific with your query parameter.'




        except Exception as err:

            print('Invalid query parameter. Ensure the spelling is correct and retry')

            raise err

    def get(self, param: 'str') -> str:

        """
            This method returns the value of any metadata parameter query if found,raises error if not found and suggest likely search pattern
            Args:
                 param: parameter to query
            return:

             str of value or dictionary if multiple match found

            exception:

             AssertionError

        """
        assert isinstance(param,
                          str), f'Argument must be a string and not {type(param)}. Try get_some() or get_all() method instead'
        metadata_dict = self._utility()

        param = param.upper()

        if param in metadata_dict:

            return metadata_dict.get(param)

        else:

            return self._incomplete_str_support(param, metadata_dict)

    def get_some(self, query_list: 'list') -> list:
        """
            This method returns the value of any metadata parameter query if found and raises error if not found
            Args:
                path: for command_line use
                query_list: list of strings of parameters to query

                values_list: Values of query found in order of argument

             :exception:
                AssertionError

        """

        try:

            assert isinstance(query_list,
                              list), 'It seems you have passed a wrong argument. The query list must be of type list'

            metadata_dict = self._utility()

            query_list = [query.upper() for query in query_list]

            values_list = []

            for queries in query_list:

                if queries in metadata_dict:

                    values_list.append(metadata_dict.get(queries))

                else:

                    values_list.append(self._incomplete_str_support(queries, metadata_dict))

            return values_list

        except Exception as err:

            raise err

    def get_all(self) -> dict:

        """
            This method returns the value of any metadata parameter query if found and raises error if not found
            args:
            path: for command_line use
            returns:

                A dictionary of all parameters in the metadata file

            :exception:

                AssertionError
        """


        return  self._utility()

###########COMMAND LINE SUPPORT####################

    def get_cli(self, param: 'str') -> str:

        """
            This method returns the value of any metadata parameter query if found,raises error if not found and suggest likely search pattern
            Args:
                 path: for command_line use
                 param: parameter to query
            return:

             str of value found

            exception:

             AssertionError

        """

        assert isinstance(param,
                          str), f'Argument must be a string and not {type(param)}. Try get_some() or get_all() method instead'
        metadata_dict = self._utility()

        param = param.upper()

        if param in metadata_dict:

            print(metadata_dict.get(param))

        else:

         print(self._incomplete_str_support(param, metadata_dict))




"""
******************************************
MODULE FOR LANDSAT FILE HANDLING
******************************************
Author1: Jolaiya Emmanuel -> jolaiyaemmanuel@gmail.com
Author2: Oke Matthew -> matthewoke16@gmail.com
License: Apache
******************************************
"""

# HOW TO USE

"""
To extract file(s) from the downloaded landsat compressed data
To perform other specific operations on landsat images.
>>> from rsgis import landsat
>>> data = landsat.extract(path_to_file)
"""
import os
import tarfile
import time
import math


class Landsat(object):
    """
    This is the class that decompresses the compressed file(s) by calling the extract function.

    Parameters:
    path(str): This is the path to the compressed file.
    name(str): This is the name of the folder where the uncompressed files will be saved.

    Result:
    str: This function returns the path to the uncompressed file.
    """

    def __init__(self, path, bands, name='extracted_data'):
        self._path, self.name, self.bands = path, name, bands
        self.tar = tarfile.open(path)
        self.path = os.path.join(os.path.dirname(path), self.name)
        self.bands_dir = {}
        self.band_searches = {}

    def _unzip(self):

        if not os.path.exists(self.path):
            os.mkdir(self.path)

        if self.bands:
            self.tar.extractall(path=self.path, members=self.band_searcher())
            self.bands_dir = self.all_bands(self.path, self.bands_dir)
            print('...almost done')
            return self.bands_dir
        else:
            self.tar.extractall(self.path)
            self.bands_dir = self.all_bands(self.path, self.bands_dir)
            print('...almost done')
            return self.bands_dir
        self.tar.close()

    def band_searcher(self):
        self.band_searches = {}
        for i in self.bands:
            if i > 0:
                self.band_searches['_B{}.TIF'.format(i)] = i
            else:
                self.band_searches['_MTL.txt'] = i
        for tarinfo in self.tar:
            if tarinfo.name.endswith(tuple(list(self.band_searches))):
                yield tarinfo

    def get_bands(self, *args):
        """
        This is used to get the path to particular landsat file(s).

        > from rsgis import landsat
        > data = landsat.extract(path_to_downloaded_file)
        > data.get_bands(1, 2)
        > [1: 'path_to_band_1', 2:'path_to_band_2']

        Parameters:
        args(int): number of band path(s) to be returned

        Returns:
        dict: Returns a dictionary containing band numbers as key and bands as values

        """

        paths = {}
        for i in args:
            try:
                paths[i] = self.bands_dir[i]
            except KeyError:
                print("The particular file/ band {} has not been unzipped".format(i))
        return paths

    def get_metadata(self):
        """
        This function is used for getting the path to only the metadata file

        > from rsgis import landsat
        > data = landsat.extract(path_to_downloaded_file)
        > data.get_metadata() =>returns the path to the metadata file

        Returns:
        str: returns the path leading to the metadata file
        """

        metadata_path = self.get_bands(0)
        metadata_path = metadata_path[0]
        return metadata_path

    def all_bands(self, path=None, bands_dir=None):
        if bands_dir is None:
            bands_dir = {}
        if path is not None:
            for root, dirs, name in os.walk(path):
                for i in name:
                    if i.endswith('_MTL.txt'):
                        bands_dir[0] = os.path.join(root, i)
                    elif i.endswith('_B10.TIF'):
                        bands_dir[10] = os.path.join(root, i)
                    elif i.endswith('_B11.TIF'):
                        bands_dir[11] = os.path.join(root, i)
                    elif i.endswith('_BQA.TIF'):
                        bands_dir[12] = os.path.join(root, i)
                    elif i.endswith('_ANG.txt'):
                        bands_dir[13] = os.path.join(root, i)
                    else:
                        bands_dir[int(i[-5])] = os.path.join(root, i)
                break
        return bands_dir


def extract(path, bands=None):
    """
    This is the function used for uncompressing files.
    Parameters:
        path (str): path to the file to be unzipped
        bands (list): list of specific bands to extract
                0 - metadata file
                1 - band1
                2 - band2, etc
    > from rsgis import landsat
    > path = '/home/user/compresion.tar.gz'
    > landsat.extract(path, bands = [0,1,2])
    """

    if bands is None:
        bands = []
    assert type(bands) == list, 'Your bands should be in a list format and not {}'.format(type(bands))
    assert all(type(i) == int for i in bands), 'bands parameter only accept integer values'
    data = Landsat(path, bands)
    print('File found. Unzipping files...')
    start_time = time.time()
    data._unzip()
    print('Task completed!')
    print('It took {:.2f} minutes to complete this operation.'.format((time.time() - start_time) / 60))
    return data
